#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

try:
  from typing import Optional, Sequence, Union
  from ..ir import *
  from ._ods_common import get_default_loc_context
  from .._mlir_libs._mlirDialectsLinalg import fill_builtin_region
except ImportError as e:
  raise RuntimeError("Error loading imports from extension module") from e

from ._ods_common import get_op_result_or_value as _get_op_result_or_value

def isa(cls: Type, ty: Type):
  try:
    cls(ty)
    return True
  except ValueError:
    return False


class InitTensorOp:
  """Extends the linalg.init_tensor op."""

  def __init__(self,
               sizes: Union[Sequence[int], Sequence[Value]],
               element_type: Type,
               *,
               loc=None,
               ip=None):
    """Constructs an `init_tensor` with either static or dynamic sizes."""
    context = get_default_loc_context(loc)
    operands = []
    attributes = {}
    # TODO: Refactor the InitTensorOp to take an element type attribute and
    # then use normal result type inference, unifying the Python and C++ side
    # with a standard mechanism (versus stashing that in builders).
    if sizes and isinstance(sizes[0], Value):
      # Dynamic sizes.
      operands.extend(sizes)
      static_size_ints = [-1] * len(sizes)
      result_type = RankedTensorType.get(static_size_ints, element_type)
    else:
      # Static sizes.
      result_type = RankedTensorType.get(sizes, element_type)
      static_size_ints = sizes

    i64_type = IntegerType.get_signless(64)
    attributes["static_sizes"] = ArrayAttr.get(
        [IntegerAttr.get(i64_type, s) for s in static_size_ints],
        context=context)
    op = self.build_generic(results=[result_type],
                            operands=operands,
                            attributes=attributes,
                            loc=loc,
                            ip=ip)
    OpView.__init__(self, op)


class StructuredOpMixin:
  """All structured ops use the same mixin class."""

  def __init__(self, inputs, outputs=(), results=(), loc=None, ip=None):
    super().__init__(
        self.build_generic(results=list(results),
                           operands=[list(inputs), list(outputs)],
                           loc=loc,
                           ip=ip))


def select_opview_mixin(parent_opview_cls):
  # TODO: This shouldn't be a heuristic: we should have a way to annotate
  # the OpView to note that it is a structured op.
  if ("__init__" not in parent_opview_cls.__dict__ and
      hasattr(parent_opview_cls, "inputs") and
      hasattr(parent_opview_cls, "outputs") and
      hasattr(parent_opview_cls, "result_tensors")):
    return StructuredOpMixin



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *

class InvalidArgsExpressionTestCase(TestBase):

    @no_debug_info_test
    def test_invalid_lang(self):
        self.expect("expression -l foo --", error=True,
                    substrs=["error: unknown language type: 'foo' for expression"])

    @no_debug_info_test
    def test_invalid_all_thread(self):
        self.expect("expression -a foo --", error=True,
                    substrs=['error: invalid all-threads value setting: "foo"'])

    @no_debug_info_test
    def test_invalid_ignore_br(self):
        self.expect("expression -i foo --", error=True,
                    substrs=['error: could not convert "foo" to a boolean value.'])

    @no_debug_info_test
    def test_invalid_allow_jit(self):
        self.expect("expression -j foo --", error=True,
                    substrs=['error: could not convert "foo" to a boolean value.'])

    @no_debug_info_test
    def test_invalid_timeout(self):
        self.expect("expression -t foo --", error=True,
                    substrs=['error: invalid timeout setting "foo"'])

        self.expect("expression -t \"\" --", error=True,
                    substrs=['error: invalid timeout setting ""'])

    @no_debug_info_test
    def test_invalid_unwind(self):
        self.expect("expression -u foo --", error=True,
                    substrs=['error: could not convert "foo" to a boolean value.'])

    @no_debug_info_test
    def test_invalid_fixits(self):
        self.expect("expression -X foo --", error=True,
                    substrs=['error: could not convert "foo" to a boolean value.'])



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import quant


def run(f):
  print("\nTEST:", f.__name__)
  f()
  return f


# CHECK-LABEL: TEST: test_type_hierarchy
@run
def test_type_hierarchy():
  with Context():
    i8 = IntegerType.get_signless(8)
    any = Type.parse("!quant.any<i8<-8:7>:f32>")
    uniform = Type.parse("!quant.uniform<i8<-8:7>:f32, 0.99872:127>")
    per_axis = Type.parse("!quant.uniform<i8:f32:1, {2.0e+2,0.99872:120}>")
    calibrated = Type.parse("!quant.calibrated<f32<-0.998:1.2321>>")

    assert not quant.QuantizedType.isinstance(i8)
    assert quant.QuantizedType.isinstance(any)
    assert quant.QuantizedType.isinstance(uniform)
    assert quant.QuantizedType.isinstance(per_axis)
    assert quant.QuantizedType.isinstance(calibrated)

    assert quant.AnyQuantizedType.isinstance(any)
    assert quant.UniformQuantizedType.isinstance(uniform)
    assert quant.UniformQuantizedPerAxisType.isinstance(per_axis)
    assert quant.CalibratedQuantizedType.isinstance(calibrated)

    assert not quant.AnyQuantizedType.isinstance(uniform)
    assert not quant.UniformQuantizedType.isinstance(per_axis)


# CHECK-LABEL: TEST: test_any_quantized_type
@run
def test_any_quantized_type():
  with Context():
    i8 = IntegerType.get_signless(8)
    f32 = F32Type.get()
    any = quant.AnyQuantizedType.get(quant.QuantizedType.FLAG_SIGNED, i8, f32,
                                     -8, 7)

    # CHECK: flags: 1
    print(f"flags: {any.flags}")
    # CHECK: signed: True
    print(f"signed: {any.is_signed}")
    # CHECK: storage type: i8
    print(f"storage type: {any.storage_type}")
    # CHECK: expressed type: f32
    print(f"expressed type: {any.expressed_type}")
    # CHECK: storage min: -8
    print(f"storage min: {any.storage_type_min}")
    # CHECK: storage max: 7
    print(f"storage max: {any.storage_type_max}")
    # CHECK: storage width: 8
    print(f"storage width: {any.storage_type_integral_width}")
    # CHECK: quantized element type: !quant.any<i8<-8:7>:f32>
    print(f"quantized element type: {any.quantized_element_type}")
    # CHECK: !quant.any<i8<-8:7>:f32>
    print(any)
    assert any == Type.parse("!quant.any<i8<-8:7>:f32>")


# CHECK-LABEL: TEST: test_uniform_type
@run
def test_uniform_type():
  with Context():
    i8 = IntegerType.get_signless(8)
    f32 = F32Type.get()
    uniform = quant.UniformQuantizedType.get(
        quant.UniformQuantizedType.FLAG_SIGNED, i8, f32, 0.99872, 127, -8, 7)

    # CHECK: scale: 0.99872
    print(f"scale: {uniform.scale}")
    # CHECK: zero point: 127
    print(f"zero point: {uniform.zero_point}")
    # CHECK: fixed point: False
    print(f"fixed point: {uniform.is_fixed_point}")
    # CHECK: !quant.uniform<i8<-8:7>:f32, 9.987200e-01:127>
    print(uniform)
    assert uniform == Type.parse("!quant.uniform<i8<-8:7>:f32, 0.99872:127>")


# CHECK-LABEL: TEST: test_uniform_per_axis_type
@run
def test_uniform_per_axis_type():
  with Context():
    i8 = IntegerType.get_signless(8)
    f32 = F32Type.get()
    per_axis = quant.UniformQuantizedPerAxisType.get(
        quant.QuantizedType.FLAG_SIGNED,
        i8,
        f32, [200, 0.99872], [0, 120],
        quantized_dimension=1,
        storage_type_min=quant.QuantizedType.default_minimum_for_integer(
            is_signed=True, integral_width=8),
        storage_type_max=quant.QuantizedType.default_maximum_for_integer(
            is_signed=True, integral_width=8))

    # CHECK: scales: None
    print(f"scales: {per_axis.scales}")
    # CHECK: zero_points: None
    print(f"zero_points: {per_axis.zero_points}")
    # CHECK: quantized dim: 1
    print(f"quantized dim: {per_axis.quantized_dimension}")
    # CHECK: fixed point: False
    print(f"fixed point: {per_axis.is_fixed_point}")
    # CHECK: !quant.uniform<i8:f32:1, {2.000000e+02,9.987200e-01:120}>
    print(per_axis)
    assert per_axis == Type.parse(
        "!quant.uniform<i8:f32:1, {2.0e+2,0.99872:120}>")


# CHECK-LABEL: TEST: test_calibrated_type
@run
def test_calibrated_type():
  with Context():
    f32 = F32Type.get()
    calibrated = quant.CalibratedQuantizedType.get(f32, -0.998, 1.2321)

    # CHECK: min: -0.998
    print(f"min: {calibrated.min}")
    # CHECK: max: 1.2321
    print(f"max: {calibrated.max}")
    # CHECK: !quant.calibrated<f32<-0.998:1.232100e+00>>
    print(calibrated)
    assert calibrated == Type.parse("!quant.calibrated<f32<-0.998:1.2321>>")



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception


class StepValueInfo(object):
    def __init__(self, step_index, watch_info, expected_value):
        self.step_index = step_index
        self.watch_info = watch_info
        self.expected_value = expected_value

    def __str__(self):
        return '{}:{}: expected value:{}'.format(self.step_index, self.watch_info, self.expected_value)

    def __eq__(self, other):
        return (self.watch_info.expression == other.watch_info.expression
                and self.expected_value == other.expected_value)

    def __hash__(self):
        return hash(self.watch_info.expression, self.expected_value)



"""
Test std::deque functionality with a decl from dbg info as content.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDbgInfoContentDeque(TestBase):

    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(self,
                                          "// Set break point at this line.",
                                          lldb.SBFileSpec("main.cpp"))

        self.runCmd("settings set target.import-std-module true")

        deque_type = "std::deque<Foo>"
        size_type = deque_type + "::size_type"
        value_type = "std::__deque_base<Foo, std::allocator<Foo> >::value_type"

        iterator_type = deque_type + "::iterator"
        iterator_children = [
            ValueCheck(name="__m_iter_"),
            ValueCheck(name="__ptr_")
        ]

        riterator_type = deque_type + "::reverse_iterator"
        riterator_children = [
            ValueCheck(name="__t"),
            ValueCheck(name="current")
        ]

        self.expect_expr("a",
                         result_type=deque_type,
                         result_children=[
                             ValueCheck(children=[ValueCheck(value="3")]),
                             ValueCheck(children=[ValueCheck(value="1")]),
                             ValueCheck(children=[ValueCheck(value="2")])
                         ])

        self.expect_expr("a.size()", result_type=size_type, result_value="3")
        self.expect_expr("a.front()",
                         result_type=value_type,
                         result_children=[ValueCheck(value="3")])
        self.expect_expr("a.back()",
                         result_type=value_type,
                         result_children=[ValueCheck(value="2")])
        self.expect_expr("a.front().a", result_type="int", result_value="3")
        self.expect_expr("a.back().a", result_type="int", result_value="2")

        self.expect("expr std::reverse(a.begin(), a.end())")
        self.expect_expr("a.front().a", result_type="int", result_value="2")
        self.expect_expr("a.back().a", result_type="int", result_value="3")

        self.expect_expr("a.begin()",
                         result_type=iterator_type,
                         result_children=iterator_children)
        self.expect_expr("a.rbegin()",
                         result_type=riterator_type,
                         result_children=riterator_children)
        self.expect_expr("a.begin()->a", result_type="int", result_value="2")
        self.expect_expr("a.rbegin()->a", result_type="int", result_value="3")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *

class GuiTestCase(TestBase):

    @no_debug_info_test
    @skipIfCursesSupportMissing
    def test_reproducer_generate_invalid_invocation(self):
        self.expect("gui blub", error=True,
                    substrs=["'gui' doesn't take any arguments."])



#!/usr/bin/env python
# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""DExTer entry point. This is the only non-module file."""

import sys

if sys.version_info < (3, 6, 0):
    sys.stderr.write("You need python 3.6 or later to run DExTer\n")
    # Equivalent to sys.exit(ReturnCode._ERROR).
    sys.exit(1)

from dex.tools import main

if __name__ == '__main__':
    return_code = main()
    sys.exit(return_code.value)



"""
Test the AArch64 SVE registers dynamic resize with multiple threads.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class RegisterCommandsTestCase(TestBase):

    def check_sve_registers(self, vg_test_value):
        z_reg_size = vg_test_value * 8
        p_reg_size = int(z_reg_size / 8)

        p_value_bytes = ['0xff', '0x55', '0x11', '0x01', '0x00']

        for i in range(32):
            s_reg_value = 's%i = 0x' % (i) + \
                ''.join('{:02x}'.format(i + 1) for _ in range(4))

            d_reg_value = 'd%i = 0x' % (i) + \
                ''.join('{:02x}'.format(i + 1) for _ in range(8))

            v_reg_value = 'v%i = 0x' % (i) + \
                ''.join('{:02x}'.format(i + 1) for _ in range(16))

            z_reg_value = '{' + \
                ' '.join('0x{:02x}'.format(i + 1)
                         for _ in range(z_reg_size)) + '}'

            self.expect("register read -f hex " + 's%i' %
                        (i), substrs=[s_reg_value])

            self.expect("register read -f hex " + 'd%i' %
                        (i), substrs=[d_reg_value])

            self.expect("register read -f hex " + 'v%i' %
                        (i), substrs=[v_reg_value])

            self.expect("register read " + 'z%i' %
                        (i), substrs=[z_reg_value])

        for i in range(16):
            p_regs_value = '{' + \
                ' '.join(p_value_bytes[i % 5] for _ in range(p_reg_size)) + '}'
            self.expect("register read " + 'p%i' % (i), substrs=[p_regs_value])

        self.expect("register read ffr", substrs=[p_regs_value])

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(['linux']))
    def test_sve_registers_dynamic_config(self):
        """Test AArch64 SVE registers multi-threaded dynamic resize. """

        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        if not self.isAArch64SVE():
            self.skipTest('SVE registers must be supported.')

        main_thread_stop_line = line_number(
            "main.c", "// Break in main thread")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", main_thread_stop_line)

        thX_break_line1 = line_number("main.c", "// Thread X breakpoint 1")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", thX_break_line1)

        thX_break_line2 = line_number("main.c", "// Thread X breakpoint 2")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", thX_break_line2)

        thY_break_line1 = line_number("main.c", "// Thread Y breakpoint 1")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", thY_break_line1)

        thY_break_line2 = line_number("main.c", "// Thread Y breakpoint 2")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", thY_break_line2)

        self.runCmd("run", RUN_SUCCEEDED)

        process = self.dbg.GetSelectedTarget().GetProcess()

        thread1 = process.GetThreadAtIndex(0)

        self.expect("thread info 1", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=["stop reason = breakpoint"])

        self.check_sve_registers(8)

        self.runCmd("process continue", RUN_SUCCEEDED)

        for idx in range(1, process.GetNumThreads()):
            thread = process.GetThreadAtIndex(idx)
            if thread.GetStopReason() != lldb.eStopReasonBreakpoint:
                self.runCmd("thread continue %d" % (idx + 1))
                self.assertEqual(thread.GetStopReason(),
                                 lldb.eStopReasonBreakpoint)

            stopped_at_line_number = thread.GetFrameAtIndex(
                0).GetLineEntry().GetLine()

            if stopped_at_line_number == thX_break_line1:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(4)
                self.runCmd('register write vg 2')

            elif stopped_at_line_number == thY_break_line1:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(2)
                self.runCmd('register write vg 4')

        self.runCmd("thread continue 2")
        self.runCmd("thread continue 3")

        for idx in range(1, process.GetNumThreads()):
            thread = process.GetThreadAtIndex(idx)
            self.assertEqual(thread.GetStopReason(),
                             lldb.eStopReasonBreakpoint)

            stopped_at_line_number = thread.GetFrameAtIndex(
                0).GetLineEntry().GetLine()

            if stopped_at_line_number == thX_break_line2:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(2)

            elif stopped_at_line_number == thY_break_line2:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(4)



#!/usr/bin/env python

# ---------------------------------------------------------------------
# Be sure to add the python path that points to the LLDB shared library.
#
# # To use this in the embedded python interpreter using "lldb" just
# import it with the full path using the "command script import"
# command
#   (lldb) command script import /path/to/cmdtemplate.py
# ---------------------------------------------------------------------

from __future__ import print_function

import inspect
import lldb
import optparse
import shlex
import sys


class FrameStatCommand:
    program = 'framestats'

    @classmethod
    def register_lldb_command(cls, debugger, module_name):
        parser = cls.create_options()
        cls.__doc__ = parser.format_help()
        # Add any commands contained in this module to LLDB
        command = 'command script add -c %s.%s %s' % (module_name,
                                                      cls.__name__,
                                                      cls.program)
        debugger.HandleCommand(command)
        print('The "{0}" command has been installed, type "help {0}" or "{0} '
              '--help" for detailed help.'.format(cls.program))

    @classmethod
    def create_options(cls):

        usage = "usage: %prog [options]"
        description = ('This command is meant to be an example of how to make '
                       'an LLDB command that does something useful, follows '
                       'best practices, and exploits the SB API. '
                       'Specifically, this command computes the aggregate '
                       'and average size of the variables in the current '
                       'frame and allows you to tweak exactly which variables '
                       'are to be accounted in the computation.')

        # Pass add_help_option = False, since this keeps the command in line
        #  with lldb commands, and we wire up "help command" to work by
        # providing the long & short help methods below.
        parser = optparse.OptionParser(
            description=description,
            prog=cls.program,
            usage=usage,
            add_help_option=False)

        parser.add_option(
            '-i',
            '--in-scope',
            action='store_true',
            dest='inscope',
            help='in_scope_only = True',
            default=True)

        parser.add_option(
            '-a',
            '--arguments',
            action='store_true',
            dest='arguments',
            help='arguments = True',
            default=True)

        parser.add_option(
            '-l',
            '--locals',
            action='store_true',
            dest='locals',
            help='locals = True',
            default=True)

        parser.add_option(
            '-s',
            '--statics',
            action='store_true',
            dest='statics',
            help='statics = True',
            default=True)

        return parser

    def get_short_help(self):
        return "Example command for use in debugging"

    def get_long_help(self):
        return self.help_string

    def __init__(self, debugger, unused):
        self.parser = self.create_options()
        self.help_string = self.parser.format_help()

    def __call__(self, debugger, command, exe_ctx, result):
        # Use the Shell Lexer to properly parse up command options just like a
        # shell would
        command_args = shlex.split(command)

        try:
            (options, args) = self.parser.parse_args(command_args)
        except:
            # if you don't handle exceptions, passing an incorrect argument to
            # the OptionParser will cause LLDB to exit (courtesy of OptParse
            # dealing with argument errors by throwing SystemExit)
            result.SetError("option parsing failed")
            return

        # Always get program state from the lldb.SBExecutionContext passed
        # in as exe_ctx
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError("invalid frame")
            return

        variables_list = frame.GetVariables(
            options.arguments,
            options.locals,
            options.statics,
            options.inscope)
        variables_count = variables_list.GetSize()
        if variables_count == 0:
            print("no variables here", file=result)
            return
        total_size = 0
        for i in range(0, variables_count):
            variable = variables_list.GetValueAtIndex(i)
            variable_type = variable.GetType()
            total_size = total_size + variable_type.GetByteSize()
            average_size = float(total_size) / variables_count
            print("Your frame has %d variables. Their total size "
                             "is %d bytes. The average size is %f bytes" % (
                                    variables_count, total_size, average_size), file=result)
        # not returning anything is akin to returning success


def __lldb_init_module(debugger, dict):
    # Register all classes that have a register_lldb_command method
    for _name, cls in inspect.getmembers(sys.modules[__name__]):
        if inspect.isclass(cls) and callable(getattr(cls,
                                                     "register_lldb_command",
                                                     None)):
            cls.register_lldb_command(debugger, __name__)



"""
Test that the lldb editline handling is configured correctly.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.lldbpexpect import PExpectTest


class EditlineTest(PExpectTest):

    @skipIfAsan
    @skipIfEditlineSupportMissing
    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])
    def test_left_right_arrow(self):
        """Test that ctrl+left/right arrow navigates words correctly.

        Note: just sending escape characters to pexpect and checking the buffer
        doesn't work well, so we run real commands. We want to type
        "help command" while exercising word-navigation, so type it as below,
        where [] indicates cursor position.

        1. Send "el rint"  -> "el rint[]"
        2. Ctrl+left once  -> "el []rint"
        3. Send "p"        -> "el p[]rint"
        4. Ctrl+left twice -> "[]el print"
        5. Send "h"        -> "h[]el print"
        6. Ctrl+right      -> "hel[] print"
        7. Send "p"        -> "help print"
        """
        self.launch()

        escape_pairs = [
            ("\x1b[1;5D", "\x1b[1;5C"),
            ("\x1b[5D", "\x1b[5C"),
            ("\x1b\x1b[D", "\x1b\x1b[C"),
        ]
        for (l_escape, r_escape) in escape_pairs:
            self.expect("el rint{L}p{L}{L}h{R}p".format(
                L=l_escape, R=r_escape), substrs=["Syntax: print"])

        self.quit()



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import arith
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects import linalg

from mlir.dialects.linalg.opdsl.lang import *


def run(f):
  print("\nTEST:", f.__name__)
  f()
  return f


# CHECK-LABEL: TEST: testInitTensor
@run
def testInitTensor():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):
      # CHECK-LABEL: func @static_sizes
      # CHECK: %0 = linalg.init_tensor [3, 4] : tensor<3x4xf32>
      @func.FuncOp.from_py_func()
      def static_sizes():
        return linalg.InitTensorOp([3, 4], f32)

      # CHECK-LABEL: func @dynamic_sizes
      # CHECK: %0 = linalg.init_tensor [%arg0, %arg1] : tensor<?x?xf32>
      @func.FuncOp.from_py_func(IndexType.get(), IndexType.get())
      def dynamic_sizes(d0, d1):
        return linalg.InitTensorOp([d0, d1], f32)

      # CHECK-LABEL: func @zero_d
      # CHECK: %0 = linalg.init_tensor [] : tensor<f32>
      @func.FuncOp.from_py_func()
      def zero_d():
        return linalg.InitTensorOp([], f32)

  print(module)


# CHECK-LABEL: TEST: testInitTensorStaticSizesAttribute
@run
def testInitTensorStaticSizesAttribute():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):
      op = linalg.InitTensorOp([3, 4], f32)
      # CHECK: [3, 4]
      print(op.attributes["static_sizes"])


# CHECK-LABEL: TEST: testFill
@run
def testFill():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):
      # CHECK-LABEL: func @fill_tensor
      #  CHECK-SAME:   %[[OUT:[0-9a-z]+]]: tensor<12x?xf32>
      #  CHECK-NEXT: %[[CST:.*]] = arith.constant 0.0{{.*}} : f32
      #  CHECK-NEXT: %[[RES:.*]] = linalg.fill ins(%[[CST]] : f32) outs(%[[OUT]] : tensor<12x?xf32>) -> tensor<12x?xf32>
      #  CHECK-NEXT: return %[[RES]] : tensor<12x?xf32>
      @func.FuncOp.from_py_func(RankedTensorType.get((12, -1), f32))
      def fill_tensor(out):
        zero = arith.ConstantOp(value=FloatAttr.get(f32, 0.), result=f32).result
        return linalg.fill(zero, outs=[out])

      # CHECK-LABEL: func @fill_buffer
      #  CHECK-SAME:   %[[OUT:[0-9a-z]+]]: memref<12x?xf32>
      #  CHECK-NEXT: %[[CST:.*]] = arith.constant 0.0{{.*}} : f32
      #  CHECK-NEXT: linalg.fill ins(%[[CST]] : f32) outs(%[[OUT]] : memref<12x?xf32>)
      #  CHECK-NEXT: return
      @func.FuncOp.from_py_func(MemRefType.get((12, -1), f32))
      def fill_buffer(out):
        zero = arith.ConstantOp(value=FloatAttr.get(f32, 0.), result=f32).result
        linalg.fill(zero, outs=[out])

  print(module)


# CHECK-LABEL: TEST: testNamedStructuredOpCustomForm
@run
def testNamedStructuredOpCustomForm():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):

      @func.FuncOp.from_py_func(
          RankedTensorType.get((4, 8), f32), RankedTensorType.get((4, 8), f32))
      def named_form(lhs, rhs):
        init_result = linalg.InitTensorOp([4, 8], f32)
        # Check for the named form with custom format
        #      CHECK: linalg.elemwise_unary
        # CHECK-SAME:    cast = #linalg.type_fn<cast_signed>
        # CHECK-SAME:    fun = #linalg.unary_fn<exp>
        # CHECK-SAME:    ins(%{{.*}} : tensor<4x8xf32>) outs(%{{.*}} : tensor<4x8xf32>)
        unary_result = linalg.elemwise_unary(lhs, outs=[init_result.result])
        #      CHECK: linalg.elemwise_binary
        # CHECK-SAME:    cast = #linalg.type_fn<cast_unsigned>
        # CHECK-SAME:    fun = #linalg.binary_fn<mul>
        # CHECK-SAME:    ins(%{{.*}}, %{{.*}} : tensor<4x8xf32>, tensor<4x8xf32>) outs(%{{.*}} : tensor<4x8xf32>)
        #      CHECK: return
        binary_result = linalg.elemwise_binary(
            lhs,
            rhs,
            outs=[init_result.result],
            fun=BinaryFn.mul,
            cast=TypeFn.cast_unsigned)
        return unary_result, binary_result

  print(module)


# CHECK-LABEL: TEST: testNamedStructuredOpGenericForm
@run
def testNamedStructuredOpGenericForm():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):

      @func.FuncOp.from_py_func(
          RankedTensorType.get((4, 16), f32), RankedTensorType.get((16, 8),
                                                                   f32))
      def named_form(lhs, rhs):
        init_result = linalg.InitTensorOp([4, 8], f32)
        #      CHECK: "linalg.matmul"(%{{.*}})
        # CHECK-NEXT:  ^bb0(%{{.*}}: f32, %{{.*}}: f32, %{{.*}}: f32):
        # CHECK-NEXT:    arith.mulf{{.*}} (f32, f32) -> f32
        # CHECK-NEXT:    arith.addf{{.*}} (f32, f32) -> f32
        # CHECK-NEXT:    linalg.yield{{.*}} (f32) -> ()
        # CHECK-NEXT:    cast = #linalg.type_fn<cast_signed>
        # CHECK-SAME:    operand_segment_sizes = dense<[2, 1]> : vector<2xi32>
        # CHECK-SAME: (tensor<4x16xf32>, tensor<16x8xf32>, tensor<4x8xf32>) -> tensor<4x8xf32>
        return linalg.matmul(lhs, rhs, outs=[init_result.result])

  module.operation.print(print_generic_op_form=True)


# CHECK-LABEL: TEST: testNamedStructuredAsGenericOp
@run
def testNamedStructuredAsGenericOp():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):

      @func.FuncOp.from_py_func(
          RankedTensorType.get((4, 16), f32), RankedTensorType.get((16, 8),
                                                                   f32))
      def generic_form(lhs, rhs):
        init_result = linalg.InitTensorOp([4, 8], f32)
        # CHECK: linalg.generic
        return linalg.matmul(
            lhs, rhs, outs=[init_result.result], emit_generic=True)

  print(module)


# CHECK-LABEL: TEST: testOpResultFromOtherOp
@run
def testOpResultFromOtherOp():
  with Context(), Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):

      @func.FuncOp.from_py_func(
          RankedTensorType.get((4, 16), f32), RankedTensorType.get((16, 8),
                                                                   f32))
      def pass_an_op_directly(arg0, arg1):
        one = arith.ConstantOp(F32Type.get(), 1.0)
        # CHECK: %[[LHS:.*]] = linalg.fill
        lhs = linalg.fill(one, outs=[arg0])
        # CHECK: %[[RHS:.*]] = linalg.fill
        rhs = linalg.fill(one, outs=[arg1])
        # CHECK: %[[INIT:.*]] = linalg.init_tensor
        init = linalg.InitTensorOp([4, 8], f32)
        # CHECK: linalg.matmul
        # CHECK: ins(%[[LHS]], %[[RHS]]
        # CHECK: outs(%[[INIT]]
        return linalg.matmul(lhs, rhs, outs=init)

  print(module)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *

class AproposTestCase(TestBase):

    @no_debug_info_test
    def test_apropos(self):
        self.expect("apropos", error=True,
                    substrs=[' must be called with exactly one argument'])
        self.expect("apropos a b", error=True,
                    substrs=[' must be called with exactly one argument'])
        self.expect("apropos ''", error=True,
                    substrs=['\'\' is not a valid search word'])

    @no_debug_info_test
    def test_apropos_variable(self):
        """Test that 'apropos variable' prints the fully qualified command name"""
        self.expect(
            'apropos variable',
            substrs=[
                'frame variable',
                'target variable',
                'watchpoint set variable'])



"""Test that C++ global variables can be inspected by name and also their mangled name."""



from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class GlobalVariablesCppTestCase(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.source = lldb.SBFileSpec('main.cpp')

    def test(self):
        self.build()

        (target, _, _, _) = lldbutil.run_to_source_breakpoint(self, "// Set break point at this line.", self.source)

        # Check that we can access g_file_global_int by its name
        self.expect("target variable g_file_global_int", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=['42'])
        self.expect("target variable abc::g_file_global_int", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=['42'])
        self.expect("target variable xyz::g_file_global_int", VARIABLES_DISPLAYED_CORRECTLY,
                    error=True, substrs=['can\'t find global variable'])

        # Check that we can access g_file_global_const_int by its name
        self.expect("target variable g_file_global_const_int", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=['1337'])
        self.expect("target variable abc::g_file_global_const_int", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=['1337'])
        self.expect("target variable xyz::g_file_global_const_int", VARIABLES_DISPLAYED_CORRECTLY,
                    error=True, substrs=['can\'t find global variable'])

        # Try accessing a global variable in anonymous namespace.
        self.expect("target variable g_anon_namespace_const_int", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=['100'])
        self.expect("target variable abc::g_anon_namespace_const_int", VARIABLES_DISPLAYED_CORRECTLY,
                    error=True, substrs=['can\'t find global variable'])
        var = target.FindFirstGlobalVariable("abc::(anonymous namespace)::g_anon_namespace_const_int")
        self.assertTrue(var.IsValid())
        self.assertEqual(var.GetName(), "abc::(anonymous namespace)::g_anon_namespace_const_int")
        self.assertEqual(var.GetValue(), "100")

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24764")
    def test_access_by_mangled_name(self):
        self.build()

        (target, _, _, _) = lldbutil.run_to_source_breakpoint(self, "// Set break point at this line.", self.source)

        # Check that we can access g_file_global_int by its mangled name
        addr = target.EvaluateExpression("&abc::g_file_global_int").GetValueAsUnsigned()
        self.assertNotEqual(addr, 0)
        mangled = lldb.SBAddress(addr, target).GetSymbol().GetMangledName()
        self.assertNotEqual(mangled, None)
        gv = target.FindFirstGlobalVariable(mangled)
        self.assertTrue(gv.IsValid())
        self.assertEqual(gv.GetName(), "abc::g_file_global_int")
        self.assertEqual(gv.GetValueAsUnsigned(), 42)



"""
Tests that auto types work
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CPPAutoTestCase(TestBase):

    def test_with_run_command(self):
        """Test that auto types work in the expression parser"""
        self.build()
        lldbutil.run_to_source_breakpoint(self, "// break here", lldb.SBFileSpec("main.cpp"))

        self.expect_expr('auto f = 123456; f', result_type='int', result_value='123456')
        self.expect(
            'expr struct Test { int x; int y; Test() : x(123), y(456) {} }; auto t = Test(); t',
            substrs=[
                'Test',
                '123',
                '456'])
        self.expect_expr("auto s = foo; s", result_type="long", result_value="1234")



"""
Test lldb data formatter subsystem.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LibCxxFunctionTestCase(TestBase):

    # Run frame var for a variable twice. Verify we do not hit the cache
    # the first time but do the second time.
    def run_frame_var_check_cache_use(self, variable, result_to_match, skip_find_function=False):
        self.runCmd("log timers reset")
        self.expect("frame variable " + variable,
                    substrs=[variable + " =  " + result_to_match])
        if not skip_find_function:
          self.expect("log timers dump",
                   substrs=["lldb_private::CompileUnit::FindFunction"])

        self.runCmd("log timers reset")
        self.expect("frame variable " + variable,
                    substrs=[variable + " =  " + result_to_match])
        self.expect("log timers dump",
                   matching=False,
                   substrs=["lldb_private::CompileUnit::FindFunction"])


    @add_test_categories(["libc++"])
    def test(self):
        """Test that std::function as defined by libc++ is correctly printed by LLDB"""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        bkpt = self.target().FindBreakpointByID(
            lldbutil.run_break_set_by_source_regexp(
                self, "Set break point at this line."))

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        self.run_frame_var_check_cache_use("foo2_f", "Lambda in File main.cpp at Line 22")

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.run_frame_var_check_cache_use("add_num2_f", "Lambda in File main.cpp at Line 13")

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.run_frame_var_check_cache_use("f2", "Lambda in File main.cpp at Line 35")
        self.run_frame_var_check_cache_use("f3", "Lambda in File main.cpp at Line 39", True)
        # TODO reenable this case when std::function formatter supports
        # general callable object case.
        #self.run_frame_var_check_cache_use("f4", "Function in File main.cpp at Line 8")

        # These cases won't hit the cache at all but also don't require
        # an expensive lookup.
        self.expect("frame variable f1",
                    substrs=['f1 =  Function = foo(int, int)'])

        self.expect("frame variable f5",
                    substrs=['f5 =  Function = Bar::add_num(int) const'])



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from dex.command.CommandBase import CommandBase
from dex.dextIR import LocIR
from dex.dextIR import ValueIR

class DexExpectStepOrder(CommandBase):
    """Expect the line every `DexExpectStepOrder` is found on to be stepped on
    in `order`. Each instance must have a set of unique ascending indicies.

    DexExpectStepOrder(*order)

    See Commands.md for more info.
    """

    def __init__(self, *args):
        if not args:
            raise TypeError('Need at least one order number')

        self.sequence = [int(x) for x in args]
        super(DexExpectStepOrder, self).__init__()

    @staticmethod
    def get_name():
        return __class__.__name__

    def eval(self, step_info):
        return {'DexExpectStepOrder': ValueIR(expression=str(step_info.current_location.lineno),
                      value=str(step_info.step_index), type_name=None,
                      error_string=None,
                      could_evaluate=True,
                      is_optimized_away=True,
                      is_irretrievable=False)}



"""
Tests that C++ member and static variables are available where they should be.
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CPPThisTestCase(TestBase):

    # rdar://problem/9962849
    @expectedFailureAll(
        compiler="gcc",
        bugnumber="llvm.org/pr15439 The 'this' pointer isn't available during expression evaluation when stopped in an inlined member function")
    @expectedFailureAll(
        compiler="icc",
        bugnumber="ICC doesn't emit correct DWARF inline debug info for inlined member functions.")
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24489: Name lookup not working correctly on Windows")
    def test_with_run_command(self):
        """Test that the appropriate member variables are available when stopped in C++ static, inline, and const methods"""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        self.set_breakpoint(line_number('main.cpp', '// breakpoint 1'))
        self.set_breakpoint(line_number('main.cpp', '// breakpoint 2'))
        self.set_breakpoint(line_number('main.cpp', '// breakpoint 3'))
        self.set_breakpoint(line_number('main.cpp', '// breakpoint 4'))

        self.runCmd("process launch", RUN_SUCCEEDED)

        self.expect("expression -- m_a = 2",
                    startstr="(int) $0 = 2")

        self.runCmd("process continue")

        # This would be disallowed if we enforced const.  But we don't.
        self.expect("expression -- m_a = 2",
                    startstr="(int) $1 = 2")

        self.expect("expression -- (int)getpid(); m_a",
                    startstr="(int) $2 = 2")

        self.runCmd("process continue")

        self.expect("expression -- s_a",
                    startstr="(int) $3 = 5")

        self.runCmd("process continue")

        self.expect("expression -- m_a",
                    startstr="(int) $4 = 2")

    def set_breakpoint(self, line):
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", line, num_expected_locations=1, loc_exact=False)



"""Test that the Objective-C syntax for dictionary/array literals and indexing works"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCNewSyntaxTest(TestBase):

    def target(self):
        return self._target

    def runToBreakpoint(self):
        self.build()
        self._target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, '// Set breakpoint 0 here.', lldb.SBFileSpec(
                'main.m', False))

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=['stopped', 'stop reason = breakpoint'])

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno = 1, expected_hit_count = 1)



"""Test that a dSYM can be found when a binary is in a bundle hnd has dots in the filename."""


#import unittest2
import os.path
from time import sleep

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


exe_name = 'find-bundle-with-dots-in-fn'  # must match Makefile

class BundleWithDotInFilenameTestCase(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.source = 'main.c'

    def tearDown(self):
        # Destroy process before TestBase.tearDown()
        self.dbg.GetSelectedTarget().GetProcess().Destroy()

        # Call super's tearDown().
        TestBase.tearDown(self)

    @skipIfRemote
    @skipUnlessDarwin
    # This test is explicitly a dSYM test, it doesn't need to run for any other config.
    @skipIf(debug_info=no_match(["dsym"]))
    def test_attach_and_check_dsyms(self):
        """Test attach to binary, see if the bundle dSYM is found"""
        exe = self.getBuildArtifact(exe_name)
        self.build()
        os.chdir(self.getBuildDir());

        # Use a file as a synchronization point between test and inferior.
        pid_file_path = lldbutil.append_to_process_working_directory(self,
            "token_pid_%d" % (int(os.getpid())))
        self.addTearDownHook(
            lambda: self.run_platform_command(
                "rm %s" %
                (pid_file_path)))

        popen = self.spawnSubprocess(exe, [pid_file_path])

        # Wait for the inferior to start up, dlopen a bundle, remove the bundle it linked in
        pid = lldbutil.wait_for_file_on_target(self, pid_file_path)

        # Since the library that was dlopen()'ed is now removed, lldb will need to find the
        # binary & dSYM via target.exec-search-paths
        settings_str = "settings set target.exec-search-paths " + self.get_process_working_directory() + "/hide.app"
        self.runCmd(settings_str)

        self.runCmd("process attach -p " + str(popen.pid))

        target = self.dbg.GetSelectedTarget()
        self.assertTrue(target.IsValid(), 'Should have a valid Target after attaching to process')

        setup_complete = target.FindFirstGlobalVariable("setup_is_complete")
        self.assertEquals(setup_complete.GetValueAsUnsigned(), 1, 'Check that inferior process has completed setup')

        # Find the bundle module, see if we found the dSYM too (they're both in "hide.app")
        i = 0
        while i < target.GetNumModules():
            mod = target.GetModuleAtIndex(i)
            if mod.GetFileSpec().GetFilename() == 'com.apple.sbd':
                dsym_name = mod.GetSymbolFileSpec().GetFilename()
                self.assertTrue (dsym_name == 'com.apple.sbd', "Check that we found the dSYM for the bundle that was loaded")
            i=i+1
        os.chdir(self.getSourceDir());



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from collections import namedtuple
from ctypes import *
from functools import partial

from .utils import *

Symbol = namedtuple("Symbol", ["num", "name", "type", "value"])

class IDebugSymbolGroup2(Structure):
  pass

class IDebugSymbolGroup2Vtbl(Structure):
  wrp = partial(WINFUNCTYPE, c_long, POINTER(IDebugSymbolGroup2))
  ids_getnumbersymbols = wrp(c_ulong_p)
  ids_getsymbolname = wrp(c_ulong, c_char_p, c_ulong, c_ulong_p)
  ids_getsymboltypename = wrp(c_ulong, c_char_p, c_ulong, c_ulong_p)
  ids_getsymbolvaluetext = wrp(c_ulong, c_char_p, c_ulong, c_ulong_p)
  _fields_ = [
      ("QueryInterface", c_void_p),
      ("AddRef", c_void_p),
      ("Release", c_void_p),
      ("GetNumberSymbols", ids_getnumbersymbols),
      ("AddSymbol", c_void_p),
      ("RemoveSymbolByName", c_void_p),
      ("RemoveSymbolByIndex", c_void_p),
      ("GetSymbolName", ids_getsymbolname),
      ("GetSymbolParameters", c_void_p),
      ("ExpandSymbol", c_void_p),
      ("OutputSymbols", c_void_p),
      ("WriteSymbol", c_void_p),
      ("OutputAsType", c_void_p),
      ("AddSymbolWide", c_void_p),
      ("RemoveSymbolByNameWide", c_void_p),
      ("GetSymbolNameWide", c_void_p),
      ("WritesymbolWide", c_void_p),
      ("OutputAsTypeWide", c_void_p),
      ("GetSymbolTypeName", ids_getsymboltypename),
      ("GetSymbolTypeNameWide", c_void_p),
      ("GetSymbolSize", c_void_p),
      ("GetSymbolOffset", c_void_p),
      ("GetSymbolRegister", c_void_p),
      ("GetSymbolValueText", ids_getsymbolvaluetext),
      ("GetSymbolValueTextWide", c_void_p),
      ("GetSymbolEntryInformation", c_void_p)
    ]

IDebugSymbolGroup2._fields_ = [("lpVtbl", POINTER(IDebugSymbolGroup2Vtbl))]

class SymbolGroup(object):
  def __init__(self, symgroup):
    self.symgroup = symgroup.contents
    self.vt = self.symgroup.lpVtbl.contents
    self.ulong = c_ulong()

  def GetNumberSymbols(self):
    res = self.vt.GetNumberSymbols(self.symgroup, byref(self.ulong))
    aborter(res, "GetNumberSymbols")
    return self.ulong.value

  def GetSymbolName(self, idx):
    buf = create_string_buffer(256)
    res = self.vt.GetSymbolName(self.symgroup, idx, buf, 255, byref(self.ulong))
    aborter(res, "GetSymbolName")
    thelen = self.ulong.value
    return string_at(buf).decode("ascii")

  def GetSymbolTypeName(self, idx):
    buf = create_string_buffer(256)
    res = self.vt.GetSymbolTypeName(self.symgroup, idx, buf, 255, byref(self.ulong))
    aborter(res, "GetSymbolTypeName")
    thelen = self.ulong.value
    return string_at(buf).decode("ascii")

  def GetSymbolValueText(self, idx, handleserror=False):
    buf = create_string_buffer(256)
    res = self.vt.GetSymbolValueText(self.symgroup, idx, buf, 255, byref(self.ulong))
    if res != 0 and handleserror:
      return None
    aborter(res, "GetSymbolTypeName")
    thelen = self.ulong.value
    return string_at(buf).decode("ascii")

  def get_symbol(self, idx):
    name = self.GetSymbolName(idx)
    thetype = self.GetSymbolTypeName(idx)
    value = self.GetSymbolValueText(idx)
    return Symbol(idx, name, thetype, value)

  def get_all_symbols(self):
    num_syms = self.GetNumberSymbols()
    return list(map(self.get_symbol, list(range(num_syms))))



"""
Test lldb-vscode completions request
"""


import lldbvscode_testcase
import unittest2
import vscode
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestVSCode_variables(lldbvscode_testcase.VSCodeTestCaseBase):

    def verify_completions(self, actual_list, expected_list, not_expected_list=[]):
        for expected_item in expected_list:
            self.assertIn(expected_item, actual_list)

        for not_expected_item in not_expected_list:
            self.assertNotIn(not_expected_item, actual_list)

    @skipIfWindows
    @skipIfDarwin # Skip this test for now until we can figure out why tings aren't working on build bots
    def test_completions(self):
        """
            Tests the completion request at different breakpoints
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 1")
        breakpoint2_line = line_number(source, "// breakpoint 2")
        breakpoint_ids = self.set_source_breakpoints(
            source, [breakpoint1_line, breakpoint2_line]
        )
        self.continue_to_next_stop()

        # shouldn't see variables inside main
        self.verify_completions(
            self.vscode.get_completions("var"),
            [
                {
                    "text": "var",
                    "label": "var -- vector<basic_string<char, char_traits<char>, allocator<char>>, allocator<basic_string<char, char_traits<char>, allocator<char>>>> &",
                }
            ],
            [{"text": "var1", "label": "var1 -- int &"}],
        )

        # should see global keywords but not variables inside main
        self.verify_completions(
            self.vscode.get_completions("str"),
            [{"text": "struct", "label": "struct"}],
            [{"text": "str1", "label": "str1 -- std::string &"}],
        )

        self.continue_to_next_stop()

        # should see variables from main but not from the other function
        self.verify_completions(
            self.vscode.get_completions("var"),
            [
                {"text": "var1", "label": "var1 -- int &"},
                {"text": "var2", "label": "var2 -- int &"},
            ],
            [
                {
                    "text": "var",
                    "label": "var -- vector<basic_string<char, char_traits<char>, allocator<char> >, allocator<basic_string<char, char_traits<char>, allocator<char> > > > &",
                }
            ],
        )

        self.verify_completions(
            self.vscode.get_completions("str"),
            [
                {"text": "struct", "label": "struct"},
                {"text": "str1", "label": "str1 -- string &"},
            ],
        )

        # should complete arbitrary commands including word starts
        self.verify_completions(
            self.vscode.get_completions("`log enable  "),
            [{"text": "gdb-remote", "label": "gdb-remote"}],
        )

        # should complete expressions with quotes inside
        self.verify_completions(
            self.vscode.get_completions('`expr " "; typed'),
            [{"text": "typedef", "label": "typedef"}],
        )

        # should complete an incomplete quoted token
        self.verify_completions(
            self.vscode.get_completions('`setting "se'),
            [
                {
                    "text": "set",
                    "label": "set -- Set the value of the specified debugger setting.",
                }
            ],
        )
        self.verify_completions(
            self.vscode.get_completions("`'comm"),
            [
                {
                    "text": "command",
                    "label": "command -- Commands for managing custom LLDB commands.",
                }
            ],
        )

        self.verify_completions(
            self.vscode.get_completions("foo1.v"),
            [
                {
                    "text": "var1",
                    "label": "foo1.var1 -- int"
                }
            ]
        )

        self.verify_completions(
            self.vscode.get_completions("foo1.my_bar_object.v"),
            [
                {
                    "text": "var1",
                    "label": "foo1.my_bar_object.var1 -- int"
                }
            ]
        )

        self.verify_completions(
            self.vscode.get_completions("foo1.var1 + foo1.v"),
            [
                {
                    "text": "var1",
                    "label": "foo1.var1 -- int"
                }
            ]
        )

        self.verify_completions(
            self.vscode.get_completions("foo1.var1 + v"),
            [
                {
                    "text": "var1",
                    "label": "var1 -- int &"
                }
            ]
        )

        #should correctly handle spaces between objects and member operators
        self.verify_completions(
            self.vscode.get_completions("foo1 .v"),
            [
                {
                    "text": "var1",
                    "label": ".var1 -- int"
                }
            ],
            [
                {
                    "text": "var2",
                    "label": ".var2 -- int"
                }
            ]
        )

        self.verify_completions(
            self.vscode.get_completions("foo1 . v"),
            [
                {
                    "text": "var1",
                    "label": "var1 -- int"
                }
            ], 
            [
                {
                    "text": "var2",
                    "label": "var2 -- int"
                }
            ]
        )



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from dex.builder.Builder import run_external_build_script
from dex.builder.ParserOptions import add_builder_tool_arguments
from dex.builder.ParserOptions import handle_builder_tool_options



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from ctypes import *
from enum import *
from functools import partial

from .utils import *
from . import control
from . import symbols
from . import sysobjs

class DebugAttach(IntFlag):
  DEBUG_ATTACH_DEFAULT =                      0
  DEBUG_ATTACH_NONINVASIVE =                  1
  DEBUG_ATTACH_EXISTING =                     2
  DEBUG_ATTACH_NONINVASIVE_NO_SUSPEND =       4
  DEBUG_ATTACH_INVASIVE_NO_INITIAL_BREAK =    8
  DEBUG_ATTACH_INVASIVE_RESUME_PROCESS =   0x10
  DEBUG_ATTACH_NONINVASIVE_ALLOW_PARTIAL = 0x20

# UUID for DebugClient7 interface.
DebugClient7IID = IID(0x13586be3, 0x542e, 0x481e, IID_Data4_Type(0xb1, 0xf2, 0x84, 0x97, 0xba, 0x74, 0xf9, 0xa9 ))

class DEBUG_CREATE_PROCESS_OPTIONS(Structure):
  _fields_ = [
    ("CreateFlags", c_ulong),
    ("EngCreateFlags", c_ulong),
    ("VerifierFlags", c_ulong),
    ("Reserved", c_ulong)
  ]

class IDebugClient7(Structure):
  pass

class IDebugClient7Vtbl(Structure):
  wrp = partial(WINFUNCTYPE, c_long, POINTER(IDebugClient7))
  idc_queryinterface = wrp(POINTER(IID), POINTER(c_void_p))
  idc_attachprocess = wrp(c_longlong, c_long, c_long)
  idc_detachprocesses = wrp()
  idc_terminateprocesses = wrp()
  idc_createprocessandattach2 = wrp(c_ulonglong, c_char_p, c_void_p, c_ulong, c_char_p, c_char_p, c_ulong, c_ulong)
  _fields_ = [
      ("QueryInterface", idc_queryinterface),
      ("AddRef", c_void_p),
      ("Release", c_void_p),
      ("AttachKernel", c_void_p),
      ("GetKernelConnectionOptions", c_void_p),
      ("SetKernelConnectionOptions", c_void_p),
      ("StartProcessServer", c_void_p),
      ("ConnectProcessServer", c_void_p),
      ("DisconnectProcessServer", c_void_p),
      ("GetRunningProcessSystemIds", c_void_p),
      ("GetRunningProcessSystemIdsByExecutableName", c_void_p),
      ("GetRunningProcessDescription", c_void_p),
      ("AttachProcess", idc_attachprocess),
      ("CreateProcess", c_void_p),
      ("CreateProcessAndAttach", c_void_p),
      ("GetProcessOptions", c_void_p),
      ("AddProcessOptions", c_void_p),
      ("RemoveProcessOptions", c_void_p),
      ("SetProcessOptions", c_void_p),
      ("OpenDumpFile", c_void_p),
      ("WriteDumpFile", c_void_p),
      ("ConnectSession", c_void_p),
      ("StartServer", c_void_p),
      ("OutputServers", c_void_p),
      ("TerminateProcesses", idc_terminateprocesses),
      ("DetachProcesses", idc_detachprocesses),
      ("EndSession", c_void_p),
      ("GetExitCode", c_void_p),
      ("DispatchCallbacks", c_void_p),
      ("ExitDispatch", c_void_p),
      ("CreateClient", c_void_p),
      ("GetInputCallbacks", c_void_p),
      ("SetInputCallbacks", c_void_p),
      ("GetOutputCallbacks", c_void_p),
      ("SetOutputCallbacks", c_void_p),
      ("GetOutputMask", c_void_p),
      ("SetOutputMask", c_void_p),
      ("GetOtherOutputMask", c_void_p),
      ("SetOtherOutputMask", c_void_p),
      ("GetOutputWidth", c_void_p),
      ("SetOutputWidth", c_void_p),
      ("GetOutputLinePrefix", c_void_p),
      ("SetOutputLinePrefix", c_void_p),
      ("GetIdentity", c_void_p),
      ("OutputIdentity", c_void_p),
      ("GetEventCallbacks", c_void_p),
      ("SetEventCallbacks", c_void_p),
      ("FlushCallbacks", c_void_p),
      ("WriteDumpFile2", c_void_p),
      ("AddDumpInformationFile", c_void_p),
      ("EndProcessServer", c_void_p),
      ("WaitForProcessServerEnd", c_void_p),
      ("IsKernelDebuggerEnabled", c_void_p),
      ("TerminateCurrentProcess", c_void_p),
      ("DetachCurrentProcess", c_void_p),
      ("AbandonCurrentProcess", c_void_p),
      ("GetRunningProcessSystemIdByExecutableNameWide", c_void_p),
      ("GetRunningProcessDescriptionWide", c_void_p),
      ("CreateProcessWide", c_void_p),
      ("CreateProcessAndAttachWide", c_void_p),
      ("OpenDumpFileWide", c_void_p),
      ("WriteDumpFileWide", c_void_p),
      ("AddDumpInformationFileWide", c_void_p),
      ("GetNumberDumpFiles", c_void_p),
      ("GetDumpFile", c_void_p),
      ("GetDumpFileWide", c_void_p),
      ("AttachKernelWide", c_void_p),
      ("GetKernelConnectionOptionsWide", c_void_p),
      ("SetKernelConnectionOptionsWide", c_void_p),
      ("StartProcessServerWide", c_void_p),
      ("ConnectProcessServerWide", c_void_p),
      ("StartServerWide", c_void_p),
      ("OutputServerWide", c_void_p),
      ("GetOutputCallbacksWide", c_void_p),
      ("SetOutputCallbacksWide", c_void_p),
      ("GetOutputLinePrefixWide", c_void_p),
      ("SetOutputLinePrefixWide", c_void_p),
      ("GetIdentityWide", c_void_p),
      ("OutputIdentityWide", c_void_p),
      ("GetEventCallbacksWide", c_void_p),
      ("SetEventCallbacksWide", c_void_p),
      ("CreateProcess2", c_void_p),
      ("CreateProcess2Wide", c_void_p),
      ("CreateProcessAndAttach2", idc_createprocessandattach2),
      ("CreateProcessAndAttach2Wide", c_void_p),
      ("PushOutputLinePrefix", c_void_p),
      ("PushOutputLinePrefixWide", c_void_p),
      ("PopOutputLinePrefix", c_void_p),
      ("GetNumberInputCallbacks", c_void_p),
      ("GetNumberOutputCallbacks", c_void_p),
      ("GetNumberEventCallbacks", c_void_p),
      ("GetQuitLockString", c_void_p),
      ("SetQuitLockString", c_void_p),
      ("GetQuitLockStringWide", c_void_p),
      ("SetQuitLockStringWide", c_void_p),
      ("SetEventContextCallbacks", c_void_p),
      ("SetClientContext", c_void_p),
    ]

IDebugClient7._fields_ = [("lpVtbl", POINTER(IDebugClient7Vtbl))]

class Client(object):
  def __init__(self):
    DbgEng = WinDLL("DbgEng")
    DbgEng.DebugCreate.argtypes = [POINTER(IID), POINTER(POINTER(IDebugClient7))]
    DbgEng.DebugCreate.restype = c_ulong

    # Call DebugCreate to create a new debug client
    ptr = POINTER(IDebugClient7)()
    res = DbgEng.DebugCreate(byref(DebugClient7IID), ptr)
    aborter(res, "DebugCreate")
    self.client = ptr.contents
    self.vt = vt = self.client.lpVtbl.contents

    def QI(iface, ptr):
      return vt.QueryInterface(self.client, byref(iface), byref(ptr))

    # Query for a control object
    ptr = c_void_p()
    res = QI(control.DebugControl7IID, ptr)
    aborter(res, "QueryInterface control")
    self.control_ptr = cast(ptr, POINTER(control.IDebugControl7))
    self.Control = control.Control(self.control_ptr)

    # Query for a SystemObjects object
    ptr = c_void_p()
    res = QI(sysobjs.DebugSystemObjects4IID, ptr)
    aborter(res, "QueryInterface sysobjects")
    self.sysobjects_ptr = cast(ptr, POINTER(sysobjs.IDebugSystemObjects4))
    self.SysObjects = sysobjs.SysObjects(self.sysobjects_ptr)

    # Query for a Symbols object
    ptr = c_void_p()
    res = QI(symbols.DebugSymbols5IID, ptr)
    aborter(res, "QueryInterface debugsymbosl5")
    self.symbols_ptr = cast(ptr, POINTER(symbols.IDebugSymbols5))
    self.Symbols = symbols.Symbols(self.symbols_ptr)

  def AttachProcess(self, pid):
    # Zero process-server id means no process-server.
    res = self.vt.AttachProcess(self.client, 0, pid, DebugAttach.DEBUG_ATTACH_DEFAULT)
    aborter(res, "AttachProcess")
    return

  def DetachProcesses(self):
    res = self.vt.DetachProcesses(self.client)
    aborter(res, "DetachProcesses")
    return

  def TerminateProcesses(self):
    res = self.vt.TerminateProcesses(self.client)
    aborter(res, "TerminateProcesses")
    return

  def CreateProcessAndAttach2(self, cmdline):
    options = DEBUG_CREATE_PROCESS_OPTIONS()
    options.CreateFlags = 0x2 # DEBUG_ONLY_THIS_PROCESS
    options.EngCreateFlags  = 0
    options.VerifierFlags = 0
    options.Reserved = 0
    attach_flags = 0
    res = self.vt.CreateProcessAndAttach2(self.client, 0, cmdline.encode("ascii"), byref(options), sizeof(options), None, None, 0, attach_flags)
    aborter(res, "CreateProcessAndAttach2")
    return



import os
import itertools
import platform
import subprocess
import sys

import lit.util
from lit.llvm import llvm_config
from lit.llvm.subst import FindTool
from lit.llvm.subst import ToolSubst


def _get_lldb_init_path(config):
    return os.path.join(config.test_exec_root, 'lit-lldb-init')


def _disallow(config, execName):
  warning = '''
    echo '*** Do not use \'{0}\' in tests; use \'%''{0}\'. ***' &&
    exit 1 && echo
  '''
  config.substitutions.append((' {0} '.format(execName),
                               warning.format(execName)))


def use_lldb_substitutions(config):
    # Set up substitutions for primary tools.  These tools must come from config.lldb_tools_dir
    # which is basically the build output directory.  We do not want to find these in path or
    # anywhere else, since they are specifically the programs which are actually being tested.

    dsname = 'debugserver' if platform.system() in ['Darwin'] else 'lldb-server'
    dsargs = [] if platform.system() in ['Darwin'] else ['gdbserver']

    build_script = os.path.dirname(__file__)
    build_script = os.path.join(build_script, 'build.py')
    build_script_args = [build_script,
                        '--compiler=any', # Default to best compiler
                        '--arch=' + str(config.lldb_bitness)]
    if config.lldb_lit_tools_dir:
        build_script_args.append('--tools-dir={0}'.format(config.lldb_lit_tools_dir))
    if config.lldb_tools_dir:
        build_script_args.append('--tools-dir={0}'.format(config.lldb_tools_dir))
    if config.llvm_libs_dir:
        build_script_args.append('--libs-dir={0}'.format(config.llvm_libs_dir))

    lldb_init = _get_lldb_init_path(config)

    primary_tools = [
        ToolSubst('%lldb',
                  command=FindTool('lldb'),
                  extra_args=['--no-lldbinit', '-S', lldb_init],
                  unresolved='fatal'),
        ToolSubst('%lldb-init',
                  command=FindTool('lldb'),
                  extra_args=['-S', lldb_init],
                  unresolved='fatal'),
        ToolSubst('%lldb-noinit',
                  command=FindTool('lldb'),
                  extra_args=['--no-lldbinit'],
                  unresolved='fatal'),
        ToolSubst('%lldb-server',
                  command=FindTool("lldb-server"),
                  extra_args=[],
                  unresolved='ignore'),
        ToolSubst('%debugserver',
                  command=FindTool(dsname),
                  extra_args=dsargs,
                  unresolved='ignore'),
        ToolSubst('%platformserver',
                  command=FindTool('lldb-server'),
                  extra_args=['platform'],
                  unresolved='ignore'),
        'lldb-test',
        'lldb-vscode',
        ToolSubst('%build',
                  command="'" + sys.executable + "'",
                  extra_args=build_script_args)
        ]

    _disallow(config, 'lldb')
    _disallow(config, 'lldb-server')
    _disallow(config, 'debugserver')
    _disallow(config, 'platformserver')

    llvm_config.add_tool_substitutions(primary_tools, [config.lldb_tools_dir])

def _use_msvc_substitutions(config):
    # If running from a Visual Studio Command prompt (e.g. vcvars), this will
    # detect the include and lib paths, and find cl.exe and link.exe and create
    # substitutions for each of them that explicitly specify /I and /L paths
    cl = lit.util.which('cl')

    if not cl:
        return

    # Don't use lit.util.which() for link.exe: In `git bash`, it will pick
    # up /usr/bin/link (another name for ln).
    link = os.path.join(os.path.dirname(cl), 'link.exe')

    cl = '"' + cl + '"'
    link = '"' + link + '"'
    includes = os.getenv('INCLUDE', '').split(';')
    libs = os.getenv('LIB', '').split(';')

    config.available_features.add('msvc')
    compiler_flags = ['"/I{}"'.format(x) for x in includes if os.path.exists(x)]
    linker_flags = ['"/LIBPATH:{}"'.format(x) for x in libs if os.path.exists(x)]

    tools = [
        ToolSubst('%msvc_cl', command=cl, extra_args=compiler_flags),
        ToolSubst('%msvc_link', command=link, extra_args=linker_flags)]
    llvm_config.add_tool_substitutions(tools)
    return

def use_support_substitutions(config):
    # Set up substitutions for support tools.  These tools can be overridden at the CMake
    # level (by specifying -DLLDB_LIT_TOOLS_DIR), installed, or as a last resort, we can use
    # the just-built version.
    host_flags = ['--target=' + config.host_triple]
    if platform.system() in ['Darwin']:
        try:
            out = subprocess.check_output(['xcrun', '--show-sdk-path']).strip()
            res = 0
        except OSError:
            res = -1
        if res == 0 and out:
            sdk_path = lit.util.to_string(out)
            llvm_config.lit_config.note('using SDKROOT: %r' % sdk_path)
            host_flags += ['-isysroot', sdk_path]
    elif sys.platform != 'win32':
        host_flags += ['-pthread']

    if sys.platform.startswith('netbsd'):
        # needed e.g. to use freshly built libc++
        host_flags += ['-L' + config.llvm_libs_dir,
                  '-Wl,-rpath,' + config.llvm_libs_dir]

    # The clang module cache is used for building inferiors.
    host_flags += ['-fmodules-cache-path={}'.format(config.clang_module_cache)]

    host_flags = ' '.join(host_flags)
    config.substitutions.append(('%clang_host', '%clang ' + host_flags))
    config.substitutions.append(('%clangxx_host', '%clangxx ' + host_flags))
    config.substitutions.append(('%clang_cl_host', '%clang_cl --target='+config.host_triple))

    additional_tool_dirs=[]
    if config.lldb_lit_tools_dir:
        additional_tool_dirs.append(config.lldb_lit_tools_dir)

    llvm_config.use_clang(additional_flags=['--target=specify-a-target-or-use-a-_host-substitution'],
                          additional_tool_dirs=additional_tool_dirs,
                          required=True, use_installed=True)


    if sys.platform == 'win32':
        _use_msvc_substitutions(config)

    have_lld = llvm_config.use_lld(additional_tool_dirs=additional_tool_dirs,
                                   required=False, use_installed=True)
    if have_lld:
        config.available_features.add('lld')


    support_tools = ['yaml2obj', 'obj2yaml', 'llvm-dwp', 'llvm-pdbutil',
                     'llvm-mc', 'llvm-readobj', 'llvm-objdump',
                     'llvm-objcopy', 'lli']
    additional_tool_dirs += [config.lldb_tools_dir, config.llvm_tools_dir]
    llvm_config.add_tool_substitutions(support_tools, additional_tool_dirs)

    _disallow(config, 'clang')

def use_lldb_repro_substitutions(config, mode):
    lldb_init = _get_lldb_init_path(config)
    substitutions = [
        ToolSubst(
            '%lldb',
            command=FindTool('lldb-repro'),
            extra_args=[mode, '--no-lldbinit', '-S', lldb_init]),
        ToolSubst(
            '%lldb-init',
            command=FindTool('lldb-repro'),
            extra_args=[mode, '-S', lldb_init]),
    ]
    llvm_config.add_tool_substitutions(substitutions, [config.lldb_tools_dir])



"""
Tests calling a function by basename
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CallCPPFunctionTestCase(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.line = line_number('main.cpp', '// breakpoint')

    def test_with_run_command(self):
        """Test calling a function by basename"""
        self.build()
        lldbutil.run_to_source_breakpoint(self, "// breakpoint", lldb.SBFileSpec("main.cpp"))

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list",
                    STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped', 'stop reason = breakpoint'])

        self.expect_expr("a_function_to_call()", result_type="int", result_value="0")



#!/usr/bin/env python3
import argparse
import os
import re
import shlex
import subprocess
import sys
import textwrap

def get_git_ref_or_rev(dir: str) -> str:
    # Run 'git symbolic-ref -q --short HEAD || git rev-parse --short HEAD'
    cmd_ref = 'git symbolic-ref -q --short HEAD'
    ref = subprocess.run(shlex.split(cmd_ref), cwd=dir, text=True,
                         stdout=subprocess.PIPE)
    if not ref.returncode:
        return ref.stdout.strip()
    cmd_rev = 'git rev-parse --short HEAD'
    return subprocess.check_output(shlex.split(cmd_rev), cwd=dir,
                                   text=True).strip()


def main():
    parser = argparse.ArgumentParser(description=textwrap.dedent('''
            This script builds two versions of BOLT (with the current and
            previous revision) and sets up symlink for llvm-bolt-wrapper.
            Passes the options through to llvm-bolt-wrapper.
            '''))
    parser.add_argument('build_dir', nargs='?', default=os.getcwd(),
                        help='Path to BOLT build directory, default is current directory')
    args, wrapper_args = parser.parse_known_args()
    bolt_path = f'{args.build_dir}/bin/llvm-bolt'

    source_dir = None
    # find the repo directory
    with open(f'{args.build_dir}/CMakeCache.txt') as f:
        for line in f:
            m = re.match(r'LLVM_SOURCE_DIR:STATIC=(.*)', line)
            if m:
                source_dir = m.groups()[0]
    if not source_dir:
        sys.exit("Source directory is not found")

    script_dir = os.path.dirname(os.path.abspath(__file__))
    wrapper_path = f'{script_dir}/llvm-bolt-wrapper.py'
    # build the current commit
    subprocess.run(shlex.split("cmake --build . --target llvm-bolt"),
                   cwd=args.build_dir)
    # rename llvm-bolt
    os.replace(bolt_path, f'{bolt_path}.new')
    # memorize the old hash for logging
    old_ref = get_git_ref_or_rev(source_dir)

    # save local changes before checkout
    subprocess.run(shlex.split("git stash"), cwd=source_dir)
    # check out the previous commit
    subprocess.run(shlex.split("git checkout -f HEAD^"), cwd=source_dir)
    # get the parent commit hash for logging
    new_ref = get_git_ref_or_rev(source_dir)
    # build the previous commit
    subprocess.run(shlex.split("cmake --build . --target llvm-bolt"),
                   cwd=args.build_dir)
    # rename llvm-bolt
    os.replace(bolt_path, f'{bolt_path}.old')
    # set up llvm-bolt-wrapper.ini
    ini = subprocess.check_output(
        shlex.split(
            f"{wrapper_path} {bolt_path}.old {bolt_path}.new") + wrapper_args,
        text=True)
    with open(f'{args.build_dir}/bin/llvm-bolt-wrapper.ini', 'w') as f:
        f.write(ini)
    # symlink llvm-bolt-wrapper
    os.symlink(wrapper_path, bolt_path)
    print(f"The repository {source_dir} has been switched from rev {old_ref} "
          f"to {new_ref}. Local changes were stashed. Switch back using\n\t"
          f"git checkout {old_ref}\n"
          "Current build directory is ready to run BOLT tests, e.g.\n\t"
          "bin/llvm-lit -sv tools/bolt/test\nor\n\t"
          "bin/llvm-lit -sv tools/bolttests")


if __name__ == "__main__":
    main()



"""Test that lldb has a default mask for addressable bits on Darwin arm64 ABI"""


import os
import re
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCorefileDefaultPtrauth(TestBase):

    @skipIf(debug_info=no_match(["dsym"]), bugnumber="This test is looking explicitly for a dSYM")
    @skipIf(archs=no_match(['arm64','arm64e']))
    @skipUnlessDarwin
    @skipIfRemote
    def test_lc_note(self):
        self.build()
        self.test_exe = self.getBuildArtifact("a.out")
        self.create_corefile = self.getBuildArtifact("create-corefile")
        self.corefile = self.getBuildArtifact("core")

        ### Create our corefile
        retcode = call(self.create_corefile + " " +  self.test_exe + " " + self.corefile, shell=True)

        ## This corefile has no metadata telling us how many bits are
        ## used for ptrauth signed function pointers.  We will need lldb
        ## to fall back on its old default value for Darwin arm64 ABIs
        ## to correctly strip the bits.

        # Create a Target with our main executable binary to get it
        # seeded in lldb's global module cache.  Then delete the Target.
        # This way when the corefile searches for a binary with its UUID,
        # it'll be found by that search.
        initial_target = self.dbg.CreateTarget(self.test_exe)
        self.dbg.DeleteTarget(initial_target)

        self.target = self.dbg.CreateTarget('')
        err = lldb.SBError()
        self.process = self.target.LoadCore(self.corefile)
        self.assertEqual(self.process.IsValid(), True)

        # target variable should show us both the actual function
        # pointer with ptrauth bits and the symbol it resolves to,
        # with the ptrauth bits stripped, e.g.
        #  (int (*)(...)) fmain = 0xe46bff0100003f90 (actual=0x0000000100003f90 a.out`main at main.c:3)

        self.expect("target variable fmain", substrs=['fmain = 0x', '(actual=0x', 'main at main.c'])



"""Test passing structs to Objective-C methods."""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjCStructArgument(TestBase):

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break inside main().
        self.main_source = "test.m"
        self.break_line = line_number(
            self.main_source, '// Set breakpoint here.')

    @add_test_categories(['pyapi'])
    @skipIf(debug_info=no_match(["gmodules"]), oslist=['ios', 'watchos', 'tvos', 'bridgeos'], archs=['armv7', 'arm64'])  # this test program only builds for ios with -gmodules
    def test_with_python_api(self):
        """Test passing structs to Objective-C methods."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        bpt = target.BreakpointCreateByLocation(
            self.main_source, self.break_line)
        self.assertTrue(bpt, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        thread_list = lldbutil.get_threads_stopped_at_breakpoint(process, bpt)

        # Make sure we stopped at the first breakpoint.
        self.assertTrue(
            len(thread_list) != 0,
            "No thread stopped at our breakpoint.")
        self.assertEquals(len(thread_list), 1,
                        "More than one thread stopped at our breakpoint.")

        frame = thread_list[0].GetFrameAtIndex(0)
        self.assertTrue(frame, "Got a valid frame 0 frame.")

        self.expect("p [summer sumThings:tts]", substrs=['9'])

        self.expect(
            "po [NSValue valueWithRect:rect]",
            substrs=['NSRect: {{0, 0}, {10, 20}}'])

        # Now make sure we can call a method that returns a struct without
        # crashing.
        cmd_value = frame.EvaluateExpression("[provider getRange]")
        self.assertTrue(cmd_value.IsValid())
